﻿using System;
using OpenTK.Graphics.OpenGL;

namespace OpenTKTest
{
    /// <summary>
    /// Represents a pre-packaged setup for blending-factor 
    /// source and destination.
    /// </summary>
    public struct BlendState
    {
        private BlendingFactorSrc _Src;
        private BlendingFactorDest _Dest;

        /// <summary>
        /// Specifies how source blending factors are computed.
        /// </summary>
        public BlendingFactorSrc Source { get { return _Src; } }

        /// <summary>
        /// Specifies how destination blending factors are computed.
        /// </summary>
        public BlendingFactorDest Dest { get { return _Dest; } }

        /// <summary>
        /// Creates a new BlendState.
        /// </summary>
        /// <param name="Source">Specifies how source blending factors are 
        /// computed.</param>
        /// <param name="Dest">Specifies how destination blending factors
        /// are computed.</param>
        public BlendState(BlendingFactorSrc Source, BlendingFactorDest Dest)
        {
            this._Src = Source;
            this._Dest = Dest;
        }

        private static BlendState _premul =
            new BlendState(BlendingFactorSrc.One, BlendingFactorDest.OneMinusSrcAlpha);

        private static BlendState _nonpremul =
            new BlendState(BlendingFactorSrc.OneMinusConstantColor,
                BlendingFactorDest.OneMinusSrcAlpha);

        private static BlendState _additive =
            new BlendState(BlendingFactorSrc.SrcAlpha,
                BlendingFactorDest.One);

        private static BlendState _multiply =
            new BlendState(BlendingFactorSrc.DstColor,
                BlendingFactorDest.OneMinusSrcAlpha);

        /// <summary>
        /// Blending for premultiplied textures.
        /// </summary>
        public static BlendState Premultiplied { get { return _premul; } }
        /// <summary>
        /// Blending for non-premultiplied textures.
        /// </summary>
        public static BlendState Normal { get { return _nonpremul; } }
        /// <summary>
        /// Additive blending, causing colors to lighten each other.
        /// </summary>
        public static BlendState Additive { get { return _additive; } }
        /// <summary>
        /// Average blending, causing colors to darken each other.
        /// </summary>
        public static BlendState Average { get { return _multiply; } }
    }
}
