﻿using System;
using System.Collections.Generic;
using System.Drawing;
using System.Drawing.Imaging;

using OpenTK;
using OpenTK.Graphics.OpenGL;

namespace OpenTKTest
{
    /// <summary>
    /// A representation of an OpenGL texture.
    /// </summary>
    public class GLTexture2D : IDisposable
    {
        private int _texID;
        private Size _size;
        private Size _realSize = Size.Empty;
        private bool _disposed = false;

        /// <summary>
        /// Generates a new instance from a <see cref="System.Drawing.Image"/>.
        /// </summary>
        /// <param name="Img">The image to load in OpenGL.</param>
        public GLTexture2D(Image Img)
        {
            _texID = GL.GenTexture();
            GL.BindTexture(TextureTarget.Texture2D, _texID);

            for (int j = 2; j <= 1024; j *= 2)
            {
                if (_realSize.Width == 0 && j >= Img.Width)
                    _realSize.Width = j;
                if (_realSize.Height == 0 && j >= Img.Height)
                    _realSize.Height = j;

                if (_realSize.Width != 0 && _realSize.Height != 0) break;
            }
            if (_realSize.Width == 0 || _realSize.Height == 0)
                throw new ArgumentOutOfRangeException();

            Bitmap realImage;
            if (_realSize == Img.Size) realImage = new Bitmap(Img);
            else
            {
                realImage = new Bitmap(_realSize.Width, _realSize.Height);
                using (Graphics g = Graphics.FromImage(realImage))
                {
                    Rectangle origRect = new Rectangle(Point.Empty,Img.Size);
                    g.DrawImage(Img, origRect, origRect, GraphicsUnit.Pixel);
                }
            }

            BitmapData bmp_data = realImage.LockBits(
                new Rectangle(Point.Empty, _realSize),
                ImageLockMode.ReadOnly,
                System.Drawing.Imaging.PixelFormat.Format32bppArgb);

            GL.TexImage2D(TextureTarget.Texture2D, 0, PixelInternalFormat.Rgba,
                bmp_data.Width, bmp_data.Height, 0,
                OpenTK.Graphics.OpenGL.PixelFormat.Bgra, PixelType.UnsignedByte,
                bmp_data.Scan0);

            realImage.UnlockBits(bmp_data);

            GL.TexParameter(TextureTarget.Texture2D, 
                TextureParameterName.TextureMinFilter, (int)TextureMinFilter.Linear);
            GL.TexParameter(TextureTarget.Texture2D, 
                TextureParameterName.TextureMagFilter, (int)TextureMagFilter.Linear);
            
            _size = Img.Size;

            realImage.Dispose();
        }

        public GLTexture2D(string filename) :
            this(Image.FromFile(filename))
        {
        }

        /// <summary>
        /// The texture's ID-number in OpenGL
        /// </summary>
        public int TexID
        {
            get
            {
                if (_disposed) throw new NullReferenceException();
                return _texID;
            }
        }

        /// <summary>
        /// The size of the image.
        /// </summary>
        public Size Size
        {
            get
            {
                if (_disposed) throw new NullReferenceException();
                return _size;
            }
        }

        /// <summary>
        /// "True" size of the image, if the original's dimensions aren't
        /// powers of 2.
        /// </summary>
        public Size RealSize
        {
            get
            {
                if (_disposed) throw new NullReferenceException();
                return _realSize;
            }
        }

        /// <summary>
        /// Proportion between the original image width and the real image width.
        /// </summary>
        public double ProportionalWidth
        {
            get { return (double)Size.Width / RealSize.Width; }
        }

        /// <summary>
        /// Proportion between the original image height and the real image height.
        /// </summary>
        public double ProportionalHeight
        {
            get { return (double)Size.Height / RealSize.Height; }
        }

        /// <summary>
        /// The width of the image.
        /// </summary>
        public int Width { get { return Size.Width; } }

        /// <summary>
        /// The height of the image.
        /// </summary>
        public int Height { get { return Size.Height; } }

        /// <summary>
        /// True if the image has been disposed; false otherwise.
        /// </summary>
        public bool Disposed { get { return _disposed; } }

        /// <summary>
        /// Disposes the image and frees it from OpenGL's memory.
        /// </summary>
        public void Dispose()
        {
            GL.DeleteTexture(_texID);
            _disposed = true;
        }
    }
}
